/*
479244 - GET_MARCACAO_ESPELHO_PONTO.SQL

Marcao de Espelho Ponto
05/06/2024

Funo: get_marcacao_espelho_ponto
Descrio: Fornece um conjunto de dados estruturado com as marcaes de ponto dos funcionrios (espelho de ponto) para
    integrao direta em consultas SQL no Zmart BI.
Retorno: Pipeline (Tabela Aninhada).
Parmetros:
    - p_nrorg:      Nmero da organizao.
    - p_dtinicio:   Data de incio do perodo de apurao (formato string).
    - p_dtfim:      Data de fim do perodo de apurao (formato string).
Observaes:
	- As datas de inicio e fim precisam corresponder ao perodo praticado pelo cliente, caso contrrio no ser
        retornado dados.
	- Cada linha representa um dia de trabalho de um funcionrio e seus respectivos registros de ponto.
	- Colunas:
		- nrvinculom: Identificador nico do vnculo empregatcio.
		- dia_referencia: Data do dia de trabalho.
		- nrperiodoapuracao: Identificador do perodo de apurao.
		- nrorg: Nmero da organizao.
		- nrcoletor: Identificador do coletor de ponto utilizado.
		- nrestrutlegal: Cdigo da estrutura legal (departamento, setor, etc.).
		- nrestrutgeren: Cdigo da estrutura gerencial.
		- hora_inicio, data_inicio, hora_fim, data_fim: Horrio e data de incio e fim do primeiro turno.
		- hora_inicio2, data_inicio2, hora_fim2, data_fim2: Horrio e data de incio e fim do segundo turno (se
            houver).
		- hora_inicio3, data_inicio3, hora_fim3, data_fim3: Horrio e data de incio e fim do terceiro turno (se
            houver).
		- hora_inicio4, data_inicio4, hora_fim4, data_fim4: Horrio e data de incio e fim do quarto turno (se houver).
		- nmvinculom: Nome do funcionrio.
		- nmvinculom_notgrouped: Nome do funcionrio (no agrupado, para uso em relatrios que no exigem agrupamento).
*/

-- Remove todas as depedncias
DROP FUNCTION get_marcacao_espelho_ponto;
/

DROP TYPE marcacao_espelho_ponto_tab;
/

-- Tipos
CREATE OR REPLACE TYPE marcacao_espelho_ponto_row AS OBJECT (
    nrvinculom              NUMBER,
    dia_referencia          DATE,
    nrperiodoapuracao       NUMBER,
    nrorg                   NUMBER,
    nrcoletor               NUMBER,
    nrestrutlegal           NUMBER,
    nrestrutgeren           NUMBER,
    hora_inicio             VARCHAR2(5),
    data_inicio             VARCHAR2(10),
    hora_fim                VARCHAR2(5),
    data_fim                VARCHAR2(10),
    hora_inicio2            VARCHAR2(5),
    data_inicio2            VARCHAR2(10),
    hora_fim2               VARCHAR2(5),
    data_fim2               VARCHAR2(10),
    hora_inicio3            VARCHAR2(5),
    data_inicio3            VARCHAR2(10),
    hora_fim3               VARCHAR2(5),
    data_fim3               VARCHAR2(10),
    hora_inicio4            VARCHAR2(5),
    data_inicio4            VARCHAR2(10),
    hora_fim4               VARCHAR2(5),
    data_fim4               VARCHAR2(10),
    nmvinculom              VARCHAR2(100)
);
/

CREATE OR REPLACE TYPE marcacao_espelho_ponto_tab AS TABLE OF marcacao_espelho_ponto_row;
/

-- Funo
CREATE OR REPLACE FUNCTION get_marcacao_espelho_ponto (
    p_nrorg     IN NUMBER,
    p_dtinicio  IN VARCHAR2,
    p_dtfim     IN VARCHAR2
) RETURN marcacao_espelho_ponto_tab PIPELINED
IS
    v_row marcacao_espelho_ponto_row := marcacao_espelho_ponto_row(
        NULL, NULL, NULL, NULL, NULL,
        NULL, NULL, NULL, NULL, NULL,
        NULL, NULL, NULL, NULL, NULL,
        NULL, NULL, NULL, NULL, NULL,
        NULL, NULL, NULL, NULL );
BEGIN
    -- Iterar sobre o cursor e popular a tabela aninhada 
    FOR registro IN (
        WITH
        marcacoes AS (
            SELECT
                marc.nrvinculom,
                marc.dthrmarcacaoponto,
                marc.dthrajustada,
                marc.iddiaocorrencia,
                marc.nrcoletor,
                pa.nrperiodoapuracao,
                marc.nrorg,
                CASE
                    WHEN marc.iddiaocorrencia IN ( 'FA', 'DA' ) THEN TRUNC( marc.dthrajustada, 'DD' ) - 1
                    WHEN marc.iddiaocorrencia IN ( 'FS', 'DS' ) THEN TRUNC( marc.dthrajustada, 'DD' ) + 1
                    ELSE                                             TRUNC( marc.dthrajustada, 'DD' )
                END AS dia_referencia
            FROM gpe_marcacaoponto marc
            JOIN pto_periodoapuracao pa ON pa.nrorg                  = marc.nrorg
                                        AND pa.dtinicialapuracao     = p_dtinicio 
                                        AND pa.dtfinalapuracao       = p_dtfim
                                        AND pa.nrtipoperiodoapuracao = ( SELECT nrtipoperiodoapuracao
                                                                         FROM pto_tipoperiodoapuracao
                                                                         WHERE nrorg          = pa.nrorg
                                                                           AND nrtipovinculom = 1 )
            WHERE marc.nrorg = p_nrorg
              AND marc.nrcoletor NOT IN (21,22,23,25)
              AND ( CASE
                        WHEN marc.iddiaocorrencia IN ('FA', 'DA') THEN TRUNC(marc.dthrajustada) - 1
                        WHEN marc.iddiaocorrencia IN ('FS', 'DS') THEN TRUNC(marc.dthrajustada) + 1
                        ELSE                                           TRUNC(marc.dthrajustada)
                    END
              ) BETWEEN pa.dtinicialapuracao AND pa.dtfinalapuracao
        ),
        ids AS (
            SELECT
                marcacoes.nrvinculom,
                marcacoes.dthrajustada,
                marcacoes.dia_referencia,
                marcacoes.nrperiodoapuracao,
                marcacoes.nrorg,
                marcacoes.nrcoletor,
                ROW_NUMBER() OVER( PARTITION BY nrvinculom, dia_referencia ORDER BY dthrajustada ) nro_marcacao
            FROM marcacoes
        ),
        res AS(
            SELECT
                *
            FROM ids
            PIVOT (
                MIN(dthrajustada)
                FOR nro_marcacao
                IN ( '1' data_inicio,
                     '2' data_fim,
                     '3' data_inicio2,
                     '4' data_fim2,
                     '5' data_inicio3,
                     '6' data_fim3,
                     '7' data_inicio4,
                     '8' data_fim4 )
            )
        )
        SELECT
            res.nrvinculom,
            res.dia_referencia, 
            res.nrperiodoapuracao, 
            res.nrorg,
            res.nrcoletor,
            vh.nrestrutlegal,
            vh.nrestrutgeren,
            to_char(res.data_inicio,  'HH24:MI'   ) AS hora_inicio,
            to_char(res.data_inicio,  'DD/MM/YYYY') AS data_inicio,
            to_char(res.data_fim,     'HH24:MI'   ) AS hora_fim,
            to_char(res.data_fim,     'DD/MM/YYYY') AS data_fim,
            to_char(res.data_inicio2, 'HH24:MI'   ) AS hora_inicio2,
            to_char(res.data_inicio2, 'DD/MM/YYYY') AS data_inicio2,
            to_char(res.data_fim2,    'HH24:MI'   ) AS hora_fim2,
            to_char(res.data_fim2,    'DD/MM/YYYY') AS data_fim2,
            to_char(res.data_inicio3, 'HH24:MI'   ) AS hora_inicio3,
            to_char(res.data_inicio3, 'DD/MM/YYYY') AS data_inicio3,
            to_char(res.data_fim3,    'HH24:MI'   ) AS hora_fim3,
            to_char(res.data_fim3,    'DD/MM/YYYY') AS data_fim3,
            to_char(res.data_inicio4, 'HH24:MI'   ) AS hora_inicio4,
            to_char(res.data_inicio4, 'DD/MM/YYYY') AS data_inicio4,
            to_char(res.data_fim4,    'HH24:MI'   ) AS hora_fim4,
            to_char(res.data_fim4,    'DD/MM/YYYY') AS data_fim4,
            vm.nmvinculom
        FROM res
        JOIN gpe_vinculom vm ON vm.nrvinculom = res.nrvinculom
                            AND vm.nrorg      = res.nrorg
        JOIN gpe_vinculoh vh ON vh.nrvinculom     = vm.nrvinculom
                            AND vh.nrorg          = vm.nrorg
                            AND vh.dtmescompetenc = ( SELECT MAX( vhh.dtmescompetenc )
                                                      FROM gpe_vinculoh vhh
                                                      WHERE vhh.dtmescompetenc <= TRUNC( TO_DATE( p_dtfim, 'DD/MM/YYYY' ), 'MM' )
                                                        AND vhh.nrorg           = vh.nrorg
                                                        AND vhh.nrvinculom      = vh.nrvinculom )
        WHERE res.nrorg             = p_nrorg
          AND res.nrcoletor NOT IN (21,22,23,25)
    ) LOOP
        v_row.nrvinculom 			:= registro.nrvinculom;
		v_row.dia_referencia 		:= registro.dia_referencia;
		v_row.nrperiodoapuracao		:= registro.nrperiodoapuracao;
		v_row.nrorg          		:= registro.nrorg;
		v_row.nrcoletor 			:= registro.nrcoletor;
		v_row.nrestrutlegal 		:= registro.nrestrutlegal;
		v_row.nrestrutgeren 		:= registro.nrestrutgeren;
		v_row.hora_inicio 			:= registro.hora_inicio;
		v_row.data_inicio 			:= registro.data_inicio;
		v_row.hora_fim 				:= registro.hora_fim;
		v_row.data_fim 				:= registro.data_fim;
		v_row.hora_inicio2 			:= registro.hora_inicio2;
		v_row.data_inicio2 			:= registro.data_inicio2;
		v_row.hora_fim2 			:= registro.hora_fim2;
		v_row.data_fim2 			:= registro.data_fim2;
		v_row.hora_inicio3 			:= registro.hora_inicio3;
		v_row.data_inicio3 			:= registro.data_inicio3;
		v_row.hora_fim3 			:= registro.hora_fim3;
		v_row.data_fim3 			:= registro.data_fim3;
		v_row.hora_inicio4 			:= registro.hora_inicio4;
		v_row.data_inicio4 			:= registro.data_inicio4;
		v_row.hora_fim4 			:= registro.hora_fim4;
		v_row.data_fim4 			:= registro.data_fim4;
		v_row.nmvinculom 			:= registro.nmvinculom;
        PIPE ROW( v_row );
    END LOOP;
END;
/

-- Exemplo de uso desta funo em um select:
--SELECT * FROM TABLE( get_marcacao_espelho_ponto( 3260, '16/01/2024', '15/02/2024' ) );
/

--SELECT * FROM TABLE( get_marcacao_espelho_ponto( :p_nrorg, :p_dtinicio, :p_dtfim ) );
/
